(**
   This is a main module of CO2.
@author Naoki Nishida
@since 2014/06/26
@since 2016/08/28
*)

let _ =
  try
    let choose (sr,u) = 
      if Array.length Sys.argv > 0
      then
	begin
	  match Sys.argv.(0) with
	    "./co2u" -> u
	  | "./co2sr" -> sr
	  | _ -> sr
	end
      else
	sr
    in
    if Array.length Sys.argv = 2
    then
      let result = Readtrs.readtrs (Sys.argv.(1)) in
      if Ctrs.has_relrew result || not (Ctrs.is_standard result)
      then raise Ctrs.Unsupported 
      else ();
      let result = Ctrs.decompose_conditions result in 
      let result = Infeasibility.rm_infeasible_rules_amap result in
(*
      if Cr.is_not_CR result
      then print_string "(COMMENT by CO3: the CTRS is not confluent)\n"
      else print_string "(COMMENT by CO3: failed to disprove confluence of the CTRS)\n";
*)

      let result2 =
	if Ctrs.is_unconditional result 
	then 
	  begin
(*
	    print_string "(COMMENT by CO3: not transformed)\n"; 
*)
	    result 
	  end
	else 
	  begin
	    try
	      let result_tmp = choose (Convert_NormalOneCTRS.convert result) in
(*
	    if Ctrs.is_WLL result 
	    then ()
	    else
		print_string "(COMMENT by CO3: Warning: the transformed TRS below is not sound for the input. )\n";
*)
	      if Convert_NormalOneCTRS.is_SR_sound result
	      then
		print_string "(COMMENT by CO3: the used transformation is sound for the CTRS)\n"
	      else
		print_string "(COMMENT by CO3: the used transformation is not sound for the CTRS)\n";
	      result_tmp
	    with
	      Ctrs.Not_normal_CTRS ->
		let result_tmp = choose (Convert_UWLLDCTRS.convert result) in
		if Convert_UWLLDCTRS.is_SR_sound result
		then
		  print_string "(COMMENT by CO3: the used transformation is sound for the CTRS)\n"
		else
		  print_string "(COMMENT by CO3: the used transformation is not sound for the CTRS)\n";
		result_tmp
	  end
      in
      begin
	print_string (Ctrs.string_of_ctrs 0 result2);
(*
	if Cr.is_CR result2
	then print_string "(COMMENT by CO3: counfluent)\n"
	else print_string "(COMMENT by CO3: failed to prove confluence)\n"
*)
(*
	if Cr.is_orthogonal result2
	then print_string "(COMMENT by CO3: orthogonal, i.e., confluent)\n"
	else
	  begin
	    print_string "(COMMENT by CO3: not orthogonal)\n";
	    if Sn.is_SN result2
	    then
	      begin
		print_string "(COMMENT by CO3: terminating)\n";
		if List.for_all (Cpair.is_joinable result2) (Cpair.all_cpairs result2)
		then
		  begin
		    print_string "(COMMENT by CO3: all CPs are joinable)\n";
		    print_string "(COMMENT by CO3: i.e., confluent)\n"
		  end
		else
		  print_string "(COMMENT by CO3: failed to prove confluence)\n"
	      end
	    else 
	      print_string "(COMMENT by CO3: failed to prove termination)\n"
	  end
*)
	()
      end
    else 
      print_string "usage: co2 <filename>\n"
  with
    Ctrs.Not_normal_CTRS -> ()
  | Ctrs.Not_oriented_3CTRS -> ()
  | Ctrs.Not_3DCTRS -> ()
  | Ctrs.Unsupported -> ()
  | Ctrs.Contradiction -> print_string "ERROR: conflicts on CONDITIONTYPE\n"
  | Failure msg -> print_string ("ERROR with Failure: "^msg^"\n")
  | _ -> print_string "ERROR: unknown\n";;
