(**
   This is a module to rewrite terms.
   @author Naoki Nishida
   @since 2014/06/27
*)
  
open Ctrs;;
  
(**
   This computes all reducts of a given term.
*)
let rec all_reducts rs =
  function
      Node(Var _,_) -> []
    | (Node(FSym f,ts) as t) ->
	let rec all_reducts_at_root =
	  function
	      [] -> []
	    | (Rew (l,r,[]))::rs1 ->
		begin
		  try
		    let theta = match_term l t in
		    (apply_subst_to_term theta r)::(all_reducts_at_root rs1)
		  with
		    Not_matching ->
		      all_reducts_at_root rs1
		end
	    | _ -> raise Unsupported
	and all_reducts_of_terms =
	  function
	      [] -> []
	    | t::ts ->
		(List.map (fun u -> u::ts) (all_reducts rs t))
		@ (List.map (fun us -> t::us) (all_reducts_of_terms ts))
	in
	(all_reducts_at_root rs)
	@ (List.map (fun us -> Node(FSym f,us)) (all_reducts_of_terms ts));;
      
(**
   This computes all normal forms of a given term.
*)
let all_nfs rs t = 
  let nfs = ref [] in
  let rec get_nfs =
    function
	[] -> !nfs
      |	(t::ts) ->
	  begin
	    match all_reducts rs t with
	      [] ->
		begin
		  nfs := t::(!nfs);
		  get_nfs ts
		end
	    | us -> get_nfs (us@ts)
	  end
  in
  get_nfs [t];;
